<?php


namespace Drupal\niobi_app\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Drupal\niobi_app\Entity\NiobiApplicationWorkflow;
use Drupal\niobi_app\Entity\NiobiApplicationWorkflowInterface;
use Drupal\niobi_app\Entity\NiobiApplication;
use Drupal\niobi_app\Entity\NiobiApplicationInterface;
use Drupal\task\TaskUtilities;
use Drupal\user\UserInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;


/**
 * Provides the route controller for niobi_group.
 *
 */
class NiobiAppController extends ControllerBase {


  /**
   * {@inheritdoc}
   */
  public function createApplicationEntity(NiobiApplicationWorkflow $application_workflow, $status = 'application') {
    $data = [
      'user_id' => \Drupal::currentUser()->id(),
      'name' => \Drupal::currentUser()->getAccountName() . ' - ' . $application_workflow->label(),
      'field_application_status' => $status,
      'field_application_workflow' => $application_workflow->id(),
      'field_current_stage' => $application_workflow->getFirstStage()->id(),
    ];
    $application = NiobiApplication::create($data);
    $application->save();
    return $application;
  }

  /**
   * {@inheritdoc}
   */
  public function createApplication(NiobiApplicationWorkflow $niobi_application_workflow) {
    $application = $this->createApplicationEntity($niobi_application_workflow, 'application');
    $application_form = $niobi_application_workflow->getFirstStage()->getApplicationForm();
    $params = [
      'niobi_form' => $application_form->id()
    ];
    $options = [
      'query' => [
        'application_id' => $application->get('uuid')->value,
      ]
    ];
    $url = Url::fromRoute('entity.niobi_form.canonical', $params, $options);
    $response = new RedirectResponse($url->toString());
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function createNomination(NiobiApplicationWorkflow $niobi_application_workflow) {
    $nomination = $this->createApplicationEntity($niobi_application_workflow, 'nomination');
    $nomination_form = $niobi_application_workflow->getFirstStage()->getNominationForm();
    $params = [
      'niobi_form' => $nomination_form->id()
    ];
    $options = [
      'query' => [
        'application_id' => $nomination->get('uuid')->value,
      ]
    ];
    $url = Url::fromRoute('entity.niobi_form.canonical', $params, $options);
    $response = new RedirectResponse($url->toString());
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function acceptNomination($uuid) {
    $check = FALSE;
    if (is_string($uuid) && (preg_match('/^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/', $uuid) === 1)) {
      $check = TRUE;
    }
    if ($check) {
      $a = \Drupal::entityTypeManager()->getStorage('niobi_application')->loadByProperties(['uuid' => $uuid]);
      $id = array_keys($a)[0];
      if ($id) {
        $application = NiobiApplication::load($id);
        $workflow = $application->getApplicationWorkflow();
        $application->set('field_application_status', 'application');
        $application->set('user_id', \Drupal::currentUser()->id());
        $application->save();
        $params = [
          'niobi_application_workflow' => $workflow->id()
        ];
        $url = Url::fromRoute('entity.niobi_application_workflow.canonical', $params);
        $response = new RedirectResponse($url->toString());
        return $response;
      }
    }

    // If we didn't get the redirect, return an error message.
    return ['#type' => 'markup' , '#markup' => '<h2>Invalid Application Code</h2>'];
  }

  /**
   * {@inheritdoc}
   */
  public function declineNomination($uuid) {
    $check = FALSE;
    if (is_string($uuid) && (preg_match('/^[0-9a-f]{8}-[0-9a-f]{4}-4[0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12}$/', $uuid) === 1)) {
      $check = TRUE;
    }
    if ($check) {
      $a = \Drupal::entityTypeManager()->getStorage('niobi_application')->loadByProperties(['uuid' => $uuid]);
      $id = array_keys($a)[0];
      if ($id) {
        $application = NiobiApplication::load($id);
        $workflow = $application->getApplicationWorkflow();
        $application->set('field_application_status', 'complete');
        $application->save();
        $params = [
          'niobi_application_workflow' => $workflow->id()
        ];
        $url = Url::fromRoute('entity.niobi_application_workflow.canonical', $params);
        $response = new RedirectResponse($url->toString());
        return $response;
      }
    }

    // If we didn't get the redirect, return an error message.
    return ['#type' => 'markup' , '#markup' => '<h2>Invalid Application Code</h2>'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitNomination(NiobiApplicationWorkflowInterface $niobi_application_workflow, NiobiApplicationInterface $niobi_application) {
    $niobi_application->set('field_application_status', 'nomination_claim');
    $niobi_application->save();
    $params = [
      'niobi_application_workflow' => $niobi_application_workflow->id()
    ];
    $url = Url::fromRoute('entity.niobi_application_workflow.canonical', $params);
    $response = new RedirectResponse($url->toString());
    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function submitApplication(NiobiApplicationWorkflowInterface $niobi_application_workflow, NiobiApplicationInterface $niobi_application) {
    // Run auto-assignment
    $this->runAutoAssignment($niobi_application);

    // Set as "in review"/
    $niobi_application->set('field_application_status', 'review');
    $niobi_application->save();

    // Redirect to application dashboard.
    $params = [
      'niobi_application_workflow' => $niobi_application_workflow->id()
    ];
    $url = Url::fromRoute('entity.niobi_application_workflow.canonical', $params);
    $response = new RedirectResponse($url->toString());
    return $response;
  }

  public function runAutoAssignment(NiobiApplicationInterface $niobi_application) {
    // Run auto-assignments.
    $stage = $niobi_application->getCurrentApplicationStage();

    $auto_assign_info = $stage->getReviewAutoAssignmentInfo();

    if ($auto_assign_info) {
      // If there is no maximum, assign to everyone.
      if (!$auto_assign_info['max'] || $auto_assign_info['pool_count'] >= $auto_assign_info['max']) {
        $reviewers = $auto_assign_info['pool']->getReviewers();
        foreach ($reviewers as $id => $reviewer) {
          // Get data for application owner
          $label_info = [
            '@reviewer' => $reviewer->label(),
            '@app' => $niobi_application->label(),
          ];
          $data = [
            'type' => 'application_review_task',
            'status' => 'active',
            'name' => t('Assign @reviewer to @app', $label_info),
            'assigned_by_type' => 'system',
            'assigned_by' => '0',
            'assigned_to' => $id,
            'assigned_to_type' => 'user',
            'field_application' => $niobi_application->id(),
            'field_review_form' => $auto_assign_info['form']->id(),
          ];
          TaskUtilities::createTask($data);
        }
      }
    }
  }

  public function DeclineReviewAssignment(UserInterface $reviewer, NiobiApplicationInterface $niobi_application) {
    return ['#markup' => ''];
  }

  public function DeclineReviewAssignmentAndMarkCOI(UserInterface $reviewer, NiobiApplicationInterface $niobi_application) {
    return ['#markup' => ''];
  }

  /**
   * {@inheritdoc}
   */
  public function submitReviewDecision(NiobiApplicationWorkflowInterface $application_workflow) {
    return ['#markup' => 'Settings page'];
  }

}
