<?php

namespace Drupal\niobi_app;

use Drupal\niobi_app\Entity\NiobiApplicationInterface;
use Drupal\user\UserInterface;

class NiobiAppUtilities {

  /**
   * @param UserInterface $reviewer
   * @param NiobiApplicationInterface $application
   * @return bool
   *   FALSE means they are not excluded, and OK to assign.
   *   TRUE means they are excluded, and should not be assigned.
   */
  public static function ExcludedAsReviewer(UserInterface $reviewer, NiobiApplicationInterface $application) {
    return FALSE;
  }

  public static function getStageName($status) {
    $stages = [
      'nomination' => 'Nomination',
      'nomination_claim' => 'Claim Nomination',
      'application' => 'Application',
      'review' => 'In Review',
      'decision' => 'Decision Pending',
      'complete' => 'Complete',
    ];

    return $stages[$status];
  }

  /**
   * @param $status
   * @param NiobiApplicationInterface $application
   * @return string
   */
  public static function generateStageList($status, NiobiApplicationInterface $application) {
    $ret = '<div class="btn-group">';
    $nomination_form = $application->getCurrentApplicationStage()->getNominationForm();
    $stages = [
      'application' => 'Application',
      'review' => 'In Review',
      'decision' => 'Decision Pending',
      'complete' => 'Complete',
    ];
    if (!empty($nomination_form)) {
      $stages = [
        'nomination' => 'Nomination',
        'nomination_claim' => 'Claim Nomination',
        'application' => 'Application',
        'review' => 'In Review',
        'decision' => 'Decision Pending',
        'complete' => 'Complete',
      ];
    }
    $stage_identified = FALSE;

    foreach ($stages as $stage => $label) {
      if ($stage === $status) {
        $ret .= '<div class="btn btn-primary">' . $label . '</div>';
        $stage_identified = TRUE;
      }
      else {
        if ($stage_identified) {
          $ret .= '<div class="btn btn-default">' . $label . '</div>';
        }
        else {
          $ret .= '<div class="btn btn-info">' . $label . '</div>';
        }
      }
    }
    $ret .= '</div>';

    return $ret;
  }

}