<?php

namespace Drupal\niobi_app\Plugin\task\Bundle;

use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\message\Entity\Message;
use Drupal\niobi_app\NiobiAppUtilities;
use Drupal\niobi_app\Entity\NiobiApplication;
use Drupal\niobi_form\Entity\NiobiForm;
use Drupal\task\Entity\Task;
use Drupal\task\Entity\TaskInterface;
use Drupal\task\TaskBundleInterface;
use Drupal\user\Entity\User;
use Drupal\Component\Render\FormattableMarkup;

/**
 * @TaskBundle(
 *   id = "application_review_task",
 *   label = @Translation("Application Review Task"),
 *   bundle = "application_review_task",
 *   system_task = TRUE
 * )
 */
class ApplicationReviewTask extends PluginBase implements TaskBundleInterface {

  /**
   * @return string
   *   A string description.
   */
  public function description()
  {
    return $this->t('Actions for  Application Review Task');
  }

  /**
   * @param $task_data
   */
  public static function createTask($task_data) {
    $reviewer = User::load($task_data['assigned_to']);
    $application = NiobiApplication::load($task_data['field_application']);
    if (!NiobiAppUtilities::ExcludedAsReviewer($reviewer, $application)) {
      $task = Task::create($task_data);
      $data = $task->get('task_data')->getValue();
      $task->save();

      // With the task created, let's add an assignment message.
      $message_data = [
        'template' => 'niobi_app_assigned_task',
        'field_assignment_verb' => 'review',
        'field_assignment_item' => $application->label(),
        'field_assignment_task' => $task,
        'field_assignment_user' => $reviewer,
      ];
      $message = Message::create($message_data);
      $message->save();

      // Now run the usual task create actions.
      if (isset($data[0]['actions']['create'])) {
        $action_data = $data[0]['actions']['create'];
        foreach ($action_data as $id => $data) {
          $plugin_manager = \Drupal::service('plugin.manager.task_action');
          $plugin_definitions = $plugin_manager->getDefinitions();
          if(isset($plugin_definitions[$id])) {
            $plugin_definitions[$id]['class']::doAction($task, $data);
          }
        }
      }
      return $task;
    }
  }

  /**
   * @param $task_data
   */
  public static function expireTask($task) {
    $data = $task->get('task_data')->getValue();
    if (isset($data[0]['actions']['expire'])) {
      $action_data = $data[0]['actions']['expire'];
      foreach ($action_data as $id => $data) {
        $plugin_manager = \Drupal::service('plugin.manager.task_action');
        $plugin_definitions = $plugin_manager->getDefinitions();
        if(isset($plugin_definitions[$id])) {
          $plugin_definitions[$id]['class']::doAction($task, $data);
        }
      }
    }
    $task->set('status', 'closed');
    $task->set('close_date', time());
    $task->set('close_type', 'completed');
    $task->save();
    return $task;
  }

  /**
   * @param TaskInterface $task
   * @return array
   */
  public static function getTaskOptions(TaskInterface $task) {
    $reviewer_id = $task->get('assigned_to')->getValue()[0]['value'];
    $app_id = $task->get('field_application')->getvalue()[0]['target_id'];
    $review_id = $task->get('field_review_form')->getvalue()[0]['target_id'];
    $review_form = NiobiForm::load($review_id);

    $url_review = $review_form->toUrl('canonical',['attributes' => ['class' => 'btn btn-success button']]);
    $link_review = Link::fromTextAndUrl('Review This Application', $url_review);
    $url_decline = Url::fromRoute('niobi_app.review_decline', ['reviewer' => $reviewer_id, 'niobi_application' => $app_id], ['attributes' => ['class' => 'btn btn-warning button']]);
    $link_decline = Link::fromTextAndUrl('Decline', $url_decline);
    $url_coi = Url::fromRoute('niobi_app.review_mark_coi', ['reviewer' => $reviewer_id, 'niobi_application' => $app_id], ['attributes' => ['class' => 'btn btn-danger button']]);
    $link_coi = Link::fromTextAndUrl('I have a conflict of interest', $url_coi);

    return ['#type' => 'markup', '#markup' => new FormattableMarkup(implode(' ', [$link_review->toString(), $link_decline->toString(), $link_coi->toString()]), [])];
  }

}
