<?php

/**
 * @file
 */

namespace Drupal\niobi_app\Plugin\views\field;

use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\niobi_app\NiobiAppUtilities;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\ResultRow;
use Drupal\webform\Entity\WebformSubmission;

/**
 * @ViewsField("niobi_app_review_assignments")
 */
class NiobiApplicationReviewAssignments extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  protected function defineOptions() {
    $options = parent::defineOptions();
    $options['view'] = ['default' => ''];
    $options['display'] = ['default' => 'default'];
    $options['arguments'] = ['default' => ''];

    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state) {
    parent::buildOptionsForm($form, $form_state);

    $form['niobi_app'] = [
      '#type' => 'details',
      '#title' => $this->t("View settings"),
      '#open' => TRUE,
    ];


  }

  /**
   * {@inheritdoc}
   */
  public function query() {
    $this->addAdditionalFields();
  }

  /**
   * {@inheritdoc}
   */
  public function render(ResultRow $values) {
    $output = NULL;
    $application = $values->_entity;
    $workflow = $application->getApplicationWorkflow();
    if ($workflow->isReadyToUse()) {
      $status = $application->get('field_application_status')->getValue()[0]['value'];
      $status_name = '<p><strong>' . NiobiAppUtilities::getStageName($status) . '</strong></p>';
      $output .= t($status_name);
    }
    else {
      \Drupal::messenger()->addError('The application has not been fully set up. Please contact the form administrator.');
      $output .= '<p><em>' . t('The application system is not fully set up.') . '</em></p>';
    }

    return ['#type' => 'markup', '#markup' => $output];
  }

}
